unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TeeComma, ExtCtrls, TeeProcs, TeEngine, Chart, Series, TeeTools,
  StdCtrls;

type
  TForm1 = class(TForm)
    Chart1: TChart;
    TeeCommander1: TTeeCommander;
    Panel1: TPanel;
    CheckBox1: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure Chart1AfterDraw(Sender: TObject);
    procedure Chart1Zoom(Sender: TObject);
    procedure Chart1UndoZoom(Sender: TObject);
  private
    { Private declarations }
    procedure ChartTool1Change(Sender: TCursorTool; x,
      y: Integer; const XValue, YValue: Double; Series: TChartSeries;
      ValueIndex: Integer);
    function InterpolateLineSeries(Series: TChartSeries;XValue: Double): Double; overload;
    function InterpolateLineSeries(Series: TChartSeries; FirstIndex,
                                  LastIndex: Integer; XValue: Double): Double; overload;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  xval: Double;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var i: Integer;
begin
  Chart1.View3D:=false;
//  Chart1.Legend.Visible

  for i:=0 to 2 do
  begin
    with Chart1.AddSeries(TLineSeries) as TLineSeries do
    begin
      FillSampleValues;
      Pointer.Visible:=true;
      Pointer.Style:=TSeriesPointerStyle(i);
    end;

    with Chart1.Tools.Add(TAnnotationTool) as TAnnotationTool do
    begin
      Shape.Font.Color:=Chart1[i].Color;
      Shape.Transparency:=50;
      Shape.Shadow.Visible:=false;
    end;
  end;

  with Chart1.Tools.Add(TCursorTool) as TCursorTool do
  begin
    Style:=cssVertical;
    //FollowMouse:=true;
    OnChange:=ChartTool1Change;
    Chart1.Draw;
    XValue:=10.5;
  end;
end;

function TForm1.InterpolateLineSeries(Series: TChartSeries;
  XValue: Double): Double;
begin
  result:=InterpolateLineSeries(Series,Series.FirstDisplayedIndex,Chart1[0].LastValueIndex,XValue);
end;

function TForm1.InterpolateLineSeries(Series: TChartSeries;
  FirstIndex, LastIndex: Integer; XValue: Double): Double;
var
  Index: Integer;
  dx,dy: Double;
begin
  for Index:=FirstIndex to LastIndex do
    if Series.XValues.Value[Index]>XValue then break;

  //safeguard
  if (Index<1) then Index:=1
  else if (Index>=Series.Count) then Index:=Series.Count-1;

  // y=(y2-y1)/(x2-x1)*(x-x1)+y1
  dx:=Series.XValues.Value[Index] - Series.XValues.Value[Index-1];
  dy:=Series.YValues.Value[Index] - Series.YValues.Value[Index-1];

  if (dx<>0) then
    result:=dy*(XValue - Series.XValues.Value[Index-1])/dx + Series.YValues.Value[Index-1]
  else result:=0;
end;

procedure TForm1.Chart1AfterDraw(Sender: TObject);
var xs, ys, i: Integer;
    yval: Double;
begin
  if CheckBox1.Checked then
  begin
    xs := Chart1.Axes.Bottom.CalcXPosValue(xval);

    for i:=0 to Chart1.SeriesCount - 1 do
    begin
      yval:=InterpolateLineSeries(Chart1[i],xval);
      ys := Chart1[i].GetVertAxis.CalcYPosValue(yval);
      if (ys>Chart1.ChartRect.Top) and (ys<Chart1.ChartRect.Bottom) then
      begin
        Chart1.Canvas.Brush.Color := Chart1[i].Color;
        Chart1.Canvas.Ellipse(xs-4,ys-4,xs+4,ys+4);
        with (Chart1.Tools[i] as TAnnotationTool) do
        begin
          Active:=true;
          Text:=SeriesTitleOrName(Chart1[i]) + ': Y('+FloatToStrF(xval, ffNumber, 8, 2)+')= ' +
              FloatToStrF(yval, ffNumber, 8, 2);
          Left:=2 + Chart1.Axes.Bottom.CalcPosValue(xval);
          Top:=Chart1.Axes.Left.CalcPosValue(yval);
        end;
      end
      else
        Chart1.Tools[i].Active:=false;
    end;
  end;
end;

procedure TForm1.ChartTool1Change(Sender: TCursorTool; x,
  y: Integer; const XValue, YValue: Double; Series: TChartSeries;
  ValueIndex: Integer);
begin
  xval := XValue;
  Chart1.Draw;
end;

procedure TForm1.Chart1Zoom(Sender: TObject);
begin
  Chart1.Draw;
end;

procedure TForm1.Chart1UndoZoom(Sender: TObject);
begin
  Chart1.Draw;
end;

end.
